#!/bin/bash
set -e

TEST_PATH="/tmp/test-p2p"
TEST_PACK_NUM=1000

# 检查是否是root权限
if [ "$EUID" -ne 0 ]; then
    echo "请以root权限运行此脚本。"
    exit 1
fi

echo "注意：测试过程执行时间较长，请耐心等候..."
echo "注意：测试过程仅会下载软件包，不会进行安装..."

# 删除测试目录 并 重新创建 进行测试 
if [ ! -f $TEST_PATH ]; then
    rm -rf $TEST_PATH
fi
mkdir -p $TEST_PATH
cd $TEST_PATH

apt update >/dev/null 2>&1

echo "开始生成要测试下载的软件包列表，花费时间较长，请稍后..."
result=($(cat /var/lib/apt/lists/localhost:30977_*:30233_kylin_V10-SP1-arm64_archive.kylinos.cn_kylin_KYLIN-ALL_dists_10.1_main_binary-arm64_Packages | grep -E '^Package:' | awk '{print $2}'))

apt_get_output=$(apt-get download "${result[@]}" 2>&1 || true)

echo "排除不能进行测试的软件包..."
error_packages=($(echo "$apt_get_output" | grep "没有源可以用来下载" | awk -F '版本的 ' '{print $2}' | awk -F ':' '{print $1}'))
packages_list=()
for element_A in "${result[@]}"; do
    if [[ ! " ${error_packages[@]} " =~ " $element_A " ]]; then
        packages_list+=("$element_A")
    fi
done

# 获取列表长度
list_length=${#packages_list[@]}

if [ "$list_length" -gt $TEST_PACK_NUM ]; then
    echo "包列表长度超过$TEST_PACK_NUM，截取前$TEST_PACK_NUM个包进行测试下载。"
    packages_list=("${packages_list[@]:0:$TEST_PACK_NUM}")
fi

echo "重置P2P缓存，准备进行测试..."
rm -rf /var/cache/apt-p2p/cache || true
rm /var/lib/apt/lists/localhost* || true
systemctl restart apt-p2p.service
apt update
# echo "监控P2P的测试状态，若下面出现，Streaming will be close字符串标识，表示P2P存在异常"
# echo "若任何其他输出则P2P功能没有问题..."

# tail -f /var/log/apt-p2p.log | grep "Streaming will be close..." > install-packages.log &
# child_pid=$!
# trap "kill $child_pid" EXIT

echo "开始进行P2P下载测试，过程执行时间较长请稍后..."
apt download ${packages_list[@]} -y